﻿using System;
using System.Data;
using System.Data.Common;
using System.Windows.Forms;

namespace Spire.DataExport.Wizard.Steps
{
    public partial class ChooseColumns : BaseUIWizardForm
    {
        public ChooseColumns()
        {
            this.WizardButton = WizardButtons.BackNextCancel;
            InitializeComponent();
        }

        private void ChooseColumns_Load(object sender, System.EventArgs e)
        {
            //this.nextCommonButton.Enabled = false;
            this.LoadColumns();
        }

        private void LoadColumns()
        {
            DataExportWizardContext context
                = this.WizardContainer.Context as DataExportWizardContext;
            DbProviderFactory factory
                = DbProviderFactories.GetFactory(context.DbProviderFacotoryName);
            using (DbConnection conn = factory.CreateConnection())
            {
                conn.ConnectionString = context.DbConnectionString;
                conn.Open();

                DbCommand command = factory.CreateCommand();
                command.Connection = conn;
                if (context.TableName != null)
                {
                    command.CommandText
                        = String.Format(" SELECT * FROM {0} ", context.TableName);
                }
                else
                {
                    command.CommandText = context.SQLCommand;
                }

                DataTable schema = null;
                try
                {
                    DbDataReader result = command.ExecuteReader();
                    schema = result.GetSchemaTable();
                }
                catch (Exception e)
                {
                    String message
                        = String.Format("Could not to acquire schema of data source.[{0}]", e.Message);
                    MessageBox.Show(message, "DataExport Wizard", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                schema.Columns.Add("ColumnSelected");
                foreach (DataRow row in schema.Rows)
                {
                    row["ColumnSelected"] = true;
                }
                this.dgvColumns.AutoGenerateColumns = false;
                this.dgvColumns.DataSource = schema;
            }
        }

        protected override bool OnNext()
        {
            bool hasColumnSelected = false;
            DataTable schema = this.dgvColumns.DataSource as DataTable;
            foreach (DataRow row in schema.Rows)
            {
                if (Convert.ToBoolean(row["ColumnSelected"]))
                {
                    hasColumnSelected = true;
                    break;
                }
            }
            if (!hasColumnSelected)
            {
                MessageBox.Show("No column will be exported.", "DataExport Wizard", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            DataExportWizardContext context
                = this.WizardContainer.Context as DataExportWizardContext;
            context.Columns = schema;

            return base.OnNext();
        }
    }
}
