﻿using System;
using System.Data;
using System.Data.Common;
using System.Windows.Forms;


namespace Spire.DataExport.Wizard.Steps
{
    public partial class ChooseDataSource : BaseUIWizardForm
    {
        public ChooseDataSource()
        {
            InitializeComponent();
            this.WizardButton = WizardButtons.BackNextCancel;
        }

        private void ChooseDataSource_Load(object sender, System.EventArgs e)
        {
            this.nextCommonButton.Enabled = false;
            this.FillDataSourceList();
        }

        private String SelectedDataSourceType
        {
            get
            {
                return this.rbDataSourceTypeTable.Checked ?
                    "Table" : this.rbDataSourceTypeView.Checked ?
                    "View" : "SQLCommand";
            }
        }

        private void FillDataSourceList()
        {
            String type = SelectedDataSourceType;
            this.gbDataSource.Text = type;

            DataExportWizardContext context
                = this.WizardContainer.Context as DataExportWizardContext;
            DbProviderFactory factory
                = DbProviderFactories.GetFactory(context.DbProviderFacotoryName);
            using (DbConnection conn = factory.CreateConnection())
            {
                conn.ConnectionString = context.DbConnectionString;
                conn.Open();

                DataTable schema = null;
                switch (type)
                {
                    case "Table":
                        schema = conn.GetSchema("Tables");
                        this.dgwDataSourceList.DataSource = schema;
                        this.dgwDataSourceList.Visible = true;
                        this.txtSQLCommand.Visible = false;
                        break;

                    case "View":
                        schema = conn.GetSchema("Views");
                        this.dgwDataSourceList.DataSource = schema;
                        this.dgwDataSourceList.Visible = true;
                        this.txtSQLCommand.Visible = false;
                        break;

                    case "SQLCommand":
                        this.dgwDataSourceList.Visible = false;
                        this.txtSQLCommand.Visible = true;
                        break;
                }
            }
        }

        private void rbDataSourceType_CheckedChanged(object sender, System.EventArgs e)
        {
            this.FillDataSourceList();
        }

        private void ChooseDataSource_Validate(object sender, System.EventArgs e)
        {
            this.nextCommonButton.Enabled = false;
            if ("SQLCommand" == this.SelectedDataSourceType)
            {
                if (this.txtSQLCommand.Text.Length == 0)
                {
                    return;
                }
            }
            else
            {
                if (this.dgwDataSourceList.SelectedRows.Count == 0)
                {
                    return;
                }
            }

            this.nextCommonButton.Enabled = true;
        }

        protected override bool OnNext()
        {
            DataExportWizardContext context
                = this.WizardContainer.Context as DataExportWizardContext;
            if ("SQLCommand" == this.SelectedDataSourceType)
            {
                context.TableName = null;
                context.SQLCommand = this.txtSQLCommand.Text;
            }
            else
            {
                DataGridViewRow row = this.dgwDataSourceList.SelectedRows[0];
                String catalog = row.Cells["TableCatalog"].Value as String;
                String schema = row.Cells["TableSchema"].Value as String;
                String name = String.Format("[{0}]", row.Cells["TableName"].Value);

                if (schema != null && schema.Length > 0)
                {
                    name = String.Format("[{0}].{1}", schema, name);
                }
                if (catalog != null && catalog.Length > 0)
                {
                    name = String.Format("[{0}].{1}", catalog, name);
                }

                context.TableName = name;
                context.SQLCommand = null;
            }

            return base.OnNext();
        }
    }
}
