using System;
using System.Data;
using System.Data.Common;
using System.Windows.Forms;

namespace Spire.DataExport.Wizard.Steps
{
    public partial class ChooseDatabase : BaseUIWizardForm
    {
        public ChooseDatabase()
        {
            InitializeComponent();
            this.WizardButton = WizardButtons.BackNextCancel;
        }

        protected override bool OnNext()
        {
            DataExportWizardContext context
                = this.WizardContainer.Context as DataExportWizardContext;
            context.DbProviderFacotoryName
                = (this.cmbDbProvider.SelectedItem as DataRowView)["InvariantName"] as String;
            context.DbConnectionString = this.txtDbConnectionString.Text;

            if (this.ValidateDatabaseConfig(context))
            {
                return base.OnNext();
            }
            return false;
        }

        private void ChooseDatabase_Load(object sender, System.EventArgs e)
        {
            //init db provider factories list
            this.cmbDbProvider.DataSource = DbProviderFactories.GetFactoryClasses();
            this.cmbDbProvider.DisplayMember = "Name";
            this.nextCommonButton.Enabled = false;
        }

        private void ChooseDatabase_Validate(object sender, System.EventArgs e)
        {
            if (this.cmbDbProvider.SelectedItem != null
                && this.txtDbConnectionString.Text.Length > 0)
            {
                this.nextCommonButton.Enabled = true;
            }
            else
            {
                this.nextCommonButton.Enabled = false;
            }
        }

        private bool ValidateDatabaseConfig(DataExportWizardContext context)
        {
            DbProviderFactory factory = null;
            try
            {
                factory = DbProviderFactories.GetFactory(context.DbProviderFacotoryName);
            }
            catch (Exception e)
            {
                String message
                    = String.Format("Can not instantiate database provider factory [{0}], [{1}].", context.DbProviderFacotoryName, e.Message);
                MessageBox.Show(message, "DataExport Wizard", MessageBoxButtons.OK, MessageBoxIcon.Error);

                return false;
            }

            try
            {
                using (DbConnection connection = factory.CreateConnection())
                {
                    connection.ConnectionString = this.txtDbConnectionString.Text;
                    connection.Open();
                }
            }
            catch (Exception e)
            {
                String message
                    = String.Format("Database connection could not be established, [{0}].", e.Message);
                MessageBox.Show(message, "DataExport Wizard", MessageBoxButtons.OK, MessageBoxIcon.Error);

                return false;
            }

            return true;
        }
    }
}