using System;
using System.Data.Common;
using System.Reflection;
using System.Windows.Forms;

namespace Spire.DataExport.Wizard.Steps
{
    public partial class ChooseResultFormat : BaseUIWizardForm
    {
        public ChooseResultFormat()
        {
            InitializeComponent();
            this.WizardButton = WizardButtons.BackFinishCancel;
            this.nextCommonButton.Enabled = false;
            foreach (Control ctrl in this.gbFormat.Controls)
            {
                if (ctrl is RadioButton)
                {
                    RadioButton rbFileType = ctrl as RadioButton;
                    rbFileType.CheckedChanged += new EventHandler(ChangeResultFileFormat);
                }
            }
        }

        private void btnOpenFile_Click(object sender, System.EventArgs e)
        {
            //store the original current directory
            String currentDirectory = System.IO.Directory.GetCurrentDirectory();

            this.dlgSaveFile.ShowDialog();

            //restore the current directory
            System.IO.Directory.SetCurrentDirectory(currentDirectory);

            //result file
            String fileName = this.dlgSaveFile.FileName;
            this.txtFileName.Text = fileName;
        }

        private void ChangeResultFileFormat(object sender, System.EventArgs e)
        {
            RadioButton rbFileType = sender as RadioButton;
            if (rbFileType.Checked)
            {
                SetResultFileFormat(rbFileType);
            }
        }

        private void SetResultFileFormat(RadioButton selectedFileType)
        {
            String fileType = selectedFileType.Name.Substring(2); //name: rbXXX
            DataExportWizardContext context
                = this.WizardContainer.Context as DataExportWizardContext;
            context.FileType = fileType;
            ChooseResultFormatValidate();
        }

        private void txtFileName_TextChanged(object sender, System.EventArgs e)
        {
            DataExportWizardContext context
                = this.WizardContainer.Context as DataExportWizardContext;
            if (this.txtFileName.Text.Length > 0)
            {
                this.nextCommonButton.Enabled = true;
                context.FileName = this.txtFileName.Text;
            }
            else
            {
                context.FileName = null;
            }
            ChooseResultFormatValidate();
        }

        private void ChooseResultFormatValidate()
        {
            this.nextCommonButton.Enabled = false;
            if (this.rbClipboard.Checked)
            {
                this.nextCommonButton.Enabled = true;
            }
            else
            {
                this.nextCommonButton.Enabled = this.txtFileName.Text.Length > 0;
            }
        }

        protected override bool OnFinish()
        {
            this.ExportData();
            return base.OnFinish();
        }

        private void ExportData()
        {
            try
            {
                DataExportWizardContext context
                    = this.WizardContainer.Context as DataExportWizardContext;
                DbProviderFactory factory
                    = DbProviderFactories.GetFactory(context.DbProviderFacotoryName);
                using (DbConnection conn = factory.CreateConnection())
                {
                    conn.ConnectionString = context.DbConnectionString;
                    conn.Open();

                    DbCommand command = factory.CreateCommand();
                    command.Connection = conn;
                    command.CommandText = context.BuildSQL();

                    String specialTypeExportMethodName
                        = String.Format("ExportData_{0}", context.FileType);
                    MethodInfo specialTypeExportMethod
                        = this.GetType().GetMethod(specialTypeExportMethodName, BindingFlags.Instance | BindingFlags.NonPublic);
                    if (specialTypeExportMethod != null)
                    {
                        specialTypeExportMethod.Invoke(this,
                            new Object[] { command, context.SelectedColumns, context.FileName });
                    }
                }
            }
            catch (Exception e)
            {
                String message
                    = String.Format("Failed to export data.[{0}]", e.Message);
                MessageBox.Show(message, "DataExport Wizard", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ExportData_XLS(DbCommand command, Spire.DataExport.Collections.StringListCollection columns, String fileName)
        {
            Spire.DataExport.XLS.CellExport cellExport
                = new Spire.DataExport.XLS.CellExport();
            cellExport.ActionAfterExport = Spire.DataExport.Common.ActionType.OpenView;
            cellExport.AutoFitColWidth = true;
            cellExport.DataFormats.CultureName = "en-US";
            cellExport.DataFormats.Currency = "#,###,##0.00";
            cellExport.DataFormats.DateTime = "yyyy-M-d H:mm";
            cellExport.DataFormats.Float = "#,###,##0.00";
            cellExport.DataFormats.Integer = "#,###,##0";
            cellExport.DataFormats.Time = "H:mm";
            cellExport.SheetOptions.AggregateFormat.Font.Name = "Arial";
            cellExport.SheetOptions.CustomDataFormat.Font.Name = "Arial";
            cellExport.SheetOptions.DefaultFont.Name = "Arial";
            cellExport.SheetOptions.FooterFormat.Font.Name = "Arial";
            cellExport.SheetOptions.HeaderFormat.Font.Name = "Arial";
            cellExport.SheetOptions.HyperlinkFormat.Font.Color = Spire.DataExport.XLS.CellColor.Blue;
            cellExport.SheetOptions.HyperlinkFormat.Font.Name = "Arial";
            cellExport.SheetOptions.HyperlinkFormat.Font.Underline = Spire.DataExport.XLS.XlsFontUnderline.Single;
            cellExport.SheetOptions.NoteFormat.Alignment.Horizontal = Spire.DataExport.XLS.HorizontalAlignment.Left;
            cellExport.SheetOptions.NoteFormat.Alignment.Vertical = Spire.DataExport.XLS.VerticalAlignment.Top;
            cellExport.SheetOptions.NoteFormat.Font.Bold = true;
            cellExport.SheetOptions.NoteFormat.Font.Name = "Tahoma";
            cellExport.SheetOptions.NoteFormat.Font.Size = 8F;
            cellExport.SheetOptions.TitlesFormat.Font.Bold = true;
            cellExport.SheetOptions.TitlesFormat.Font.Name = "Arial";
            cellExport.Columns = columns;
            cellExport.SQLCommand = command;

            cellExport.FileName = fileName;
            cellExport.SaveToFile();
        }

        private void ExportData_PDF(DbCommand command, Spire.DataExport.Collections.StringListCollection columns, String fileName)
        {
            Spire.DataExport.PDF.PDFExport pdfExport
                = new Spire.DataExport.PDF.PDFExport();
            pdfExport.ActionAfterExport = Spire.DataExport.Common.ActionType.OpenView;
            pdfExport.DataFormats.CultureName = "en-US";
            pdfExport.DataFormats.Currency = "#,###,##0.00";
            pdfExport.DataFormats.DateTime = "yyyy-M-d H:mm";
            pdfExport.DataFormats.Float = "#,###,##0.00";
            pdfExport.DataFormats.Integer = "#,###,##0";
            pdfExport.DataFormats.Time = "H:mm";
            pdfExport.PDFOptions.PageOptions.Format = Spire.DataExport.PDF.PageFormat.User;
            pdfExport.PDFOptions.PageOptions.Height = 11.67;
            pdfExport.PDFOptions.PageOptions.MarginBottom = 0.78;
            pdfExport.PDFOptions.PageOptions.MarginLeft = 1.17;
            pdfExport.PDFOptions.PageOptions.MarginRight = 0.57;
            pdfExport.PDFOptions.PageOptions.MarginTop = 0.78;
            pdfExport.PDFOptions.PageOptions.Width = 8.25;
            pdfExport.Columns = columns;
            pdfExport.SQLCommand = command;

            pdfExport.FileName = fileName;
            pdfExport.SaveToFile();
        }

        private void ExportData_HTML(DbCommand command, Spire.DataExport.Collections.StringListCollection columns, String fileName)
        {
            Spire.DataExport.HTML.HTMLExport htmlExport
                = new Spire.DataExport.HTML.HTMLExport();
            htmlExport.ActionAfterExport = Spire.DataExport.Common.ActionType.OpenView;
            htmlExport.DataFormats.CultureName = "en-US";
            htmlExport.DataFormats.Currency = "#,###,##0.00";
            htmlExport.DataFormats.DateTime = "yyyy-M-d H:mm";
            htmlExport.DataFormats.Float = "#,###,##0.00";
            htmlExport.DataFormats.Integer = "#,###,##0";
            htmlExport.DataFormats.Time = "H:mm";
            htmlExport.HtmlStyle = Spire.DataExport.HTML.HtmlStyle.MSMoney;
            htmlExport.HtmlTextOptions.Font = new System.Drawing.Font("Arial", 8F);

            htmlExport.Columns = columns;
            htmlExport.SQLCommand = command;

            htmlExport.FileName = fileName;
            htmlExport.SaveToFile();
        }

        private void ExportData_RTF(DbCommand command, Spire.DataExport.Collections.StringListCollection columns, String fileName)
        {
            Spire.DataExport.RTF.RTFExport rtfExport
                = new Spire.DataExport.RTF.RTFExport();
            rtfExport.ActionAfterExport = Spire.DataExport.Common.ActionType.OpenView;
            rtfExport.DataFormats.CultureName = "en-US";
            rtfExport.DataFormats.Currency = "#,###,##0.00";
            rtfExport.DataFormats.DateTime = "yyyy-M-d H:mm";
            rtfExport.DataFormats.Float = "#,###,##0.00";
            rtfExport.DataFormats.Integer = "#,###,##0";
            rtfExport.DataFormats.Time = "H:mm";
            rtfExport.RTFOptions.DataStyle.Font = new System.Drawing.Font("Arial", 10F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.World);
            rtfExport.RTFOptions.FooterStyle.Font = new System.Drawing.Font("Arial", 10F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.World);
            rtfExport.RTFOptions.HeaderStyle.Font = new System.Drawing.Font("Arial", 10F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.World);
            rtfExport.RTFOptions.TitleStyle.Alignment = Spire.DataExport.RTF.RtfTextAlignment.Center;
            rtfExport.RTFOptions.TitleStyle.Font = new System.Drawing.Font("Arial", 10F, System.Drawing.FontStyle.Bold);

            rtfExport.Columns = columns;
            rtfExport.SQLCommand = command;

            rtfExport.FileName = fileName;
            rtfExport.SaveToFile();
        }

        private void ExportData_Clipboard(DbCommand command, Spire.DataExport.Collections.StringListCollection columns, String fileName)
        {
            Spire.DataExport.Clipboard.ClipboardExport clipboardExport
                = new Spire.DataExport.Clipboard.ClipboardExport();
            clipboardExport.ShowContent = true;
            clipboardExport.DataFormats.CultureName = "en-US";
            clipboardExport.DataFormats.Currency = "#,###,##0.00";
            clipboardExport.DataFormats.DateTime = "yyyy-M-d H:mm";
            clipboardExport.DataFormats.Float = "#,###,##0.00";
            clipboardExport.DataFormats.Integer = "#,###,##0";
            clipboardExport.DataFormats.Time = "H:mm";

            clipboardExport.Columns = columns;
            clipboardExport.SQLCommand = command;

            clipboardExport.SaveToFile();
        }

        private void ExportData_DBF(DbCommand command, Spire.DataExport.Collections.StringListCollection columns, String fileName)
        {
            Spire.DataExport.DBF.DBFExport dbfExport
                = new Spire.DataExport.DBF.DBFExport();
            dbfExport.ActionAfterExport = Spire.DataExport.Common.ActionType.OpenView;

            dbfExport.Columns = columns;
            dbfExport.SQLCommand = command;

            dbfExport.FileName = fileName;
            dbfExport.SaveToFile();
        }

        private void ExportData_MSAccess(DbCommand command, Spire.DataExport.Collections.StringListCollection columns, String fileName)
        {
            Spire.DataExport.Access.AccessExport accessExport
                = new Spire.DataExport.Access.AccessExport();
            accessExport.DataFormats.CultureName = "en-US";
            accessExport.DataFormats.Currency = "#,###,##0.00";
            accessExport.DataFormats.DateTime = "yyyy-M-d H:mm";
            accessExport.DataFormats.Float = "#,###,##0.00";
            accessExport.DataFormats.Integer = "#,###,##0";
            accessExport.DataFormats.Time = "H:mm";
            accessExport.TableName = "ExportData";

            accessExport.Columns = columns;
            accessExport.SQLCommand = command;

            accessExport.DatabaseName = fileName;
            accessExport.SaveToFile();
        }

        private void ExportData_XML(DbCommand command, Spire.DataExport.Collections.StringListCollection columns, String fileName)
        {
            Spire.DataExport.XML.XMLExport xmlExport
                = new Spire.DataExport.XML.XMLExport();
            xmlExport.ActionAfterExport = Spire.DataExport.Common.ActionType.OpenView;
            xmlExport.DataFormats.CultureName = "en-US";
            xmlExport.DataFormats.Currency = "#,###,##0.00";
            xmlExport.DataFormats.DateTime = "yyyy-M-d H:mm";
            xmlExport.DataFormats.Float = "#,###,##0.00";
            xmlExport.DataFormats.Integer = "#,###,##0";
            xmlExport.DataFormats.Time = "H:mm";

            xmlExport.Columns = columns;
            xmlExport.SQLCommand = command;

            xmlExport.FileName = fileName;
            xmlExport.SaveToFile();
        }

        private void ExportData_SQLScript(DbCommand command, Spire.DataExport.Collections.StringListCollection columns, String fileName)
        {
            Spire.DataExport.SQL.SQLExport sqlExport
                = new Spire.DataExport.SQL.SQLExport();
            sqlExport.ActionAfterExport = Spire.DataExport.Common.ActionType.OpenView;
            sqlExport.CommitStatement = "COMMIT;";
            sqlExport.DataFormats.CultureName = "en-US";
            sqlExport.DataFormats.Currency = "#,###,##0.00";
            sqlExport.DataFormats.DateTime = "yyyy-M-d H:mm";
            sqlExport.DataFormats.Float = "#,###,##0.00";
            sqlExport.DataFormats.Integer = "#,###,##0";
            sqlExport.DataFormats.Time = "H:mm";
            sqlExport.TableName = "TestTable";

            sqlExport.Columns = columns;
            sqlExport.SQLCommand = command;

            sqlExport.FileName = fileName;
            sqlExport.SaveToFile();
        }

        private void ExportData_SYLK(DbCommand command, Spire.DataExport.Collections.StringListCollection columns, String fileName)
        {
            Spire.DataExport.TXT.TXTExport txtExport
                = new Spire.DataExport.TXT.TXTExport();
            txtExport.ActionAfterExport = Spire.DataExport.Common.ActionType.OpenView;
            txtExport.DataFormats.CultureName = "en-US";
            txtExport.DataFormats.Currency = "#,###,##0.00";
            txtExport.DataFormats.DateTime = "yyyy-M-d H:mm";
            txtExport.DataFormats.Float = "#,###,##0.00";
            txtExport.DataFormats.Integer = "#,###,##0";
            txtExport.DataFormats.Time = "H:mm";
            txtExport.DataEncoding = Spire.DataExport.Common.EncodingType.ASCII;
            txtExport.ExportType = Spire.DataExport.TXT.TextExportType.SYLK;

            txtExport.Columns = columns;
            txtExport.SQLCommand = command;

            txtExport.FileName = fileName;
            txtExport.SaveToFile();
        }

        private void ExportData_CSV(DbCommand command, Spire.DataExport.Collections.StringListCollection columns, String fileName)
        {
            Spire.DataExport.TXT.TXTExport txtExport
                = new Spire.DataExport.TXT.TXTExport();
            txtExport.ActionAfterExport = Spire.DataExport.Common.ActionType.OpenView;
            txtExport.DataFormats.CultureName = "en-US";
            txtExport.DataFormats.Currency = "#,###,##0.00";
            txtExport.DataFormats.DateTime = "yyyy-M-d H:mm";
            txtExport.DataFormats.Float = "#,###,##0.00";
            txtExport.DataFormats.Integer = "#,###,##0";
            txtExport.DataFormats.Time = "H:mm";
            txtExport.ExportType = Spire.DataExport.TXT.TextExportType.CSV;

            txtExport.Columns = columns;
            txtExport.SQLCommand = command;

            txtExport.FileName = fileName;
            txtExport.SaveToFile();
        }

        private void ExportData_DIF(DbCommand command, Spire.DataExport.Collections.StringListCollection columns, String fileName)
        {
            Spire.DataExport.TXT.TXTExport txtExport
                = new Spire.DataExport.TXT.TXTExport();
            txtExport.ActionAfterExport = Spire.DataExport.Common.ActionType.OpenView;
            txtExport.DataFormats.CultureName = "en-US";
            txtExport.DataFormats.Currency = "#,###,##0.00";
            txtExport.DataFormats.DateTime = "yyyy-M-d H:mm";
            txtExport.DataFormats.Float = "#,###,##0.00";
            txtExport.DataFormats.Integer = "#,###,##0";
            txtExport.DataFormats.Time = "H:mm";
            txtExport.ExportType = Spire.DataExport.TXT.TextExportType.DIF;

            txtExport.Columns = columns;
            txtExport.SQLCommand = command;

            txtExport.FileName = fileName;
            txtExport.SaveToFile();
        }

        private void ExportData_Text(DbCommand command, Spire.DataExport.Collections.StringListCollection columns, String fileName)
        {
            Spire.DataExport.TXT.TXTExport txtExport
                = new Spire.DataExport.TXT.TXTExport();
            txtExport.ActionAfterExport = Spire.DataExport.Common.ActionType.OpenView;
            txtExport.DataFormats.CultureName = "en-US";
            txtExport.DataFormats.Currency = "#,###,##0.00";
            txtExport.DataFormats.DateTime = "yyyy-M-d H:mm";
            txtExport.DataFormats.Float = "#,###,##0.00";
            txtExport.DataFormats.Integer = "#,###,##0";
            txtExport.DataFormats.Time = "H:mm";
            txtExport.ExportType = Spire.DataExport.TXT.TextExportType.TXT;

            txtExport.Columns = columns;
            txtExport.SQLCommand = command;

            txtExport.FileName = fileName;
            txtExport.SaveToFile();
        }

        private void ChooseResultFormat_Load(object sender, System.EventArgs e)
        {
            foreach (Control ctrl in this.gbFormat.Controls)
            {
                if (ctrl is RadioButton)
                {
                    RadioButton rbFileType = ctrl as RadioButton;
                    if (rbFileType.Checked)
                    {
                        SetResultFileFormat(rbFileType);
                    }
                }
            }
        }
    }
}