using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;

namespace Spire.DataExport.Wizard
{
    public class WizardController : Component
    {
        private List<BaseWizardForm> formList = new List<BaseWizardForm>();
        protected int currentStep = 0;

        protected IWin32Window parentWindow = null;

        #region Constructor and Destructor
        public WizardController()
            : base()
        {

        }

        #endregion

        #region IDisposable Members

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                foreach (BaseWizardForm form in formList)
                {
                    if (!form.IsDisposed)
                        form.Dispose();
                }
            }

            base.Dispose(disposing);
        }

        #endregion

        #region Methods



        public void AddForm(BaseWizardForm form)
        {
            this.formList.Add(form);
            form.WizardContainer = this;
        }

        public void AddForm(Type formType)
        {
            BaseWizardForm form = Activator.CreateInstance(formType) as BaseWizardForm;
            this.AddForm(form);
        }

        public void RemoveForm(BaseWizardForm form)
        {
            this.formList.Remove(form);
        }

        public DialogResult ShowDialog(IWin32Window parent)
        {
            this.parentWindow = parent;

            return this.ShowDialog();
        }

        protected virtual DialogResult ShowDialog()
        {
            DialogResult ret = this.formList[this.currentStep].ShowDialog(this.parentWindow);
            if (ret == DialogResult.OK)
            {
                if (this.NextStep())
                    return this.ShowDialog();
            }
            else if (ret == DialogResult.Retry)
            {
                if (this.PreStep())
                    return this.ShowDialog();
            }
            else
            {
                return DialogResult.Cancel;
            }

            return ret;
        }

        protected virtual bool NextStep()
        {
            if (this.currentStep == this.formList.Count - 1)
                return false;

            this.currentStep++;

            return true;
        }

        protected virtual bool PreStep()
        {
            if (this.currentStep == 0)
                return false;

            this.currentStep--;

            return true;
        }

        private Object context;
        public Object Context
        {
            get
            {
                return this.context;
            }
            set
            {
                this.context = value;
            }
        }
        #endregion

    }
}
